from .command import Command, CommandResult, CompositeCommand
from aamp_app.devices.dummy_heater import DummyHeater


class DummyHeaterParentCommand(Command):
    """Parent class for all DummyHeater commands."""

    receiver_cls = DummyHeater

    def __init__(self, receiver: DummyHeater, **kwargs):
        super().__init__(receiver, **kwargs)


class DummyHeaterInitialize(DummyHeaterParentCommand):
    """Initialize the heater by setting to room temp (25 C)."""

    def __init__(self, receiver: DummyHeater, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class DummyHeaterDeinitialize(DummyHeaterParentCommand):
    """Deinitialize the heater by setting to room temp (25 C)."""

    def __init__(self, receiver: DummyHeater, reset_init_flag: bool = True, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["reset_init_flag"] = reset_init_flag

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.deinitialize(self._params["reset_init_flag"])
        )


class DummyHeaterSetHeatRate(DummyHeaterParentCommand):
    """Set the heating rate of the heater."""

    def __init__(self, receiver: DummyHeater, heat_rate: float, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["heat_rate"] = heat_rate

    def execute(self) -> None:
        self._receiver.heat_rate = self._params["heat_rate"]

        # logic implemented here because heat_rate setter does not return Tuple[bool, str]
        if self._receiver.heat_rate == self._params["heat_rate"]:
            self._result = CommandResult(
                True,
                "Successfully set DummyHeater heat rate to "
                + str(self._receiver.heat_rate),
            )
        else:
            self._result = CommandResult(
                False,
                "Failed to set DummyHeater heat rate. Heat rate is currently "
                + str(self._receiver.heat_rate),
            )


class DummyHeaterSetTemp(DummyHeaterParentCommand):
    """Set the temperature of the heater and wait for it to stabilize."""

    def __init__(self, receiver: DummyHeater, temperature: float, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["temperature"] = temperature

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.set_temp(self._params["temperature"])
        )


# Composite command example
class DummyHeaterRampHoldRamp(CompositeCommand):
    """Ramps up to temperature1 @ rate1, holds for specified time, then ramps to temperature2 @ rate2"""

    def __init__(
        self,
        receiver: DummyHeater,
        temp1: float,
        rate1: float,
        hold_time: float,
        temp2: float,
        rate2: float,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.add_command(DummyHeaterSetHeatRate(receiver, rate1))
        self.add_command(DummyHeaterSetTemp(receiver, temp1))
        self.add_command(DummyHeaterSetHeatRate(receiver, rate2, delay=hold_time))
        self.add_command(DummyHeaterSetTemp(receiver, temp2))



from .command import Command, CommandResult
from aamp_app.devices.multi_stepper import MultiStepper


class MultiStepperParentCommand(Command):
    """Parent class for all MultiStepper commands."""

    receiver_cls = MultiStepper

    def __init__(self, receiver: MultiStepper, **kwargs):
        super().__init__(receiver, **kwargs)


class MultiStepperConnect(MultiStepperParentCommand):
    """Open the one serial port for all stepper's controlled by a single arduino."""

    def __init__(self, receiver: MultiStepper, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.start_serial())


class MultiStepperInitialize(MultiStepperParentCommand):
    """Initialize all passed steppers by homing them."""

    def __init__(self, receiver: MultiStepper, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class MultiStepperDeinitialize(MultiStepperParentCommand):
    """Deinitialize all passed steppers by zeroing them."""

    def __init__(self, receiver: MultiStepper, reset_init_flag: bool = True, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["reset_init_flag"] = reset_init_flag

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.deinitialize(self._params["reset_init_flag"])
        )


class MultiStepperMoveAbsolute(MultiStepperParentCommand):
    """Move stepper to absolute position."""

    def __init__(
        self, receiver: MultiStepper, stepper_number: int, position: float, **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["stepper_number"] = stepper_number
        self._params["position"] = position

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.move_absolute(
                self._params["stepper_number"], self._params["position"]
            )
        )


class MultiStepperMoveRelative(MultiStepperParentCommand):
    """Move stepper by relative distance."""

    def __init__(
        self, receiver: MultiStepper, stepper_number: int, distance: float, **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["stepper_number"] = stepper_number
        self._params["distance"] = distance

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.move_relative(
                self._params["stepper_number"], self._params["distance"]
            )
        )



from typing import Tuple, List
from aamp_app.devices.oxygen_sensor import OxygenSensor
from .command import Command, CommandResult


class OxygenParentCommand(Command):
    """Parent class for DFRobot Oxygen Sensor Commands"""

    receiver_cls = OxygenSensor

    def __init__(self, receiver: OxygenSensor, **kwargs):
        super().__init__(receiver, **kwargs)


class OxygenConnect(Command):
    """Open the serial port to the oxygen sensor"""

    receiver_cls = OxygenSensor

    def __init__(self, receiver: OxygenSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.start_serial())


class OxygenInitialize(OxygenParentCommand):
    """Initialize the oxygen sensor"""

    def __init__(self, receiver: OxygenSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class OxygenDeinitialize(OxygenParentCommand):
    """Deinitialize the oxygen sensor"""

    def __init__(self, receiver: OxygenSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.deinitialize())


class OxygenGetOxygen(OxygenParentCommand):
    """Read average oxygen data"""

    def __init__(self, receiver: OxygenSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.get_oxygen())



from typing import List

from aamp_app.devices.device import Device
from .command import Command, CommandResult, CompositeCommand
from aamp_app.devices.keithley_2450 import Keithley2450


class KeithleyParentCommand(Command):
    """Parent class for all Keithley2450 commands."""

    receiver_cls = Keithley2450

    def __init__(self, receiver: Keithley2450, **kwargs):
        super().__init__(receiver, **kwargs)


class Keithley2450Initialize(KeithleyParentCommand):
    """Initialize the SMU by resetting it"""

    def __init__(self, receiver: Keithley2450, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class Keithley2450Deinitialize(KeithleyParentCommand):
    """Deinitialize the SMU"""

    # TODO: create command to deinitialize keithley2450 if necessary

    def __init__(self, reciever: Keithley2450, reset_init_flag: bool = True, **kwargs):
        super().__init__(reciever, **kwargs)
        self._params["reset_init_flag"] = reset_init_flag

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.deinitialize(self._params["reset_init_flag"])
        )


class KeithleyWait(KeithleyParentCommand):
    """Wait for all pending operations to finish"""

    def __init__(self, receiver: Keithley2450, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.wait())


class KeithleyWriteCommand(KeithleyParentCommand):
    """Write arbitrary SCPI ASCII command"""

    def __init__(self, receiver: Keithley2450, command: str, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["command"] = command

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.write_command(self._params["command"])
        )


class KeithleySetTerminal(KeithleyParentCommand):
    """Set the terminal position of the SMU"""

    def __init__(self, receiver: Keithley2450, position: str = "front", **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["position"] = position

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.terminal_pos(self._params["position"])
        )


class KeithleyErrorCheck(KeithleyParentCommand):
    """Check for errors that occur during measurement"""

    def __init__(self, receiver: Keithley2450, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.error_check())


class KeithleyClearBuffer(KeithleyParentCommand):
    """Clear the data storage buffer within the Keithley2450"""

    def __init__(self, receiver: Keithley2450, buffer: str = "defbuffer1", **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["buffer"] = buffer

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.clear_buffer(self._params["buffer"])
        )


class KeithleyIVCharacteristic(KeithleyParentCommand):
    """I-V linear sweep sourcing voltage and measuring current"""

    def __init__(
        self,
        receiver: Keithley2450,
        ilimit: float,
        vmin: float,
        vmax: float,
        delay: float,
        steps: int = 60,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["ilimit"] = ilimit
        self._params["vmin"] = vmin
        self._params["vmax"] = vmax
        self._params["delay"] = delay
        self._params["steps"] = steps

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.IV_characteristic(
                self._params["ilimit"],
                self._params["vmin"],
                self._params["vmax"],
                self._params["steps"],
                self._params["delay"],
            )
        )


class KeithleyFourPoint(KeithleyParentCommand):
    """Four collinear point sheet resistance measurement"""

    def __init__(
        self,
        receiver: Keithley2450,
        test_curr: float,
        vlimit: float,
        curr_reversal: bool = False,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["test_curr"] = test_curr
        self._params["vlimit"] = vlimit
        self._params["curr_reversal"] = curr_reversal

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.four_point(
                self._params["test_curr"],
                self._params["vlimit"],
                self._params["curr_reversal"],
            )
        )


class KeithleyGetData(KeithleyParentCommand):
    """Retrieve data"""

    def __init__(
        self,
        receiver: Keithley2450,
        filename: str = None,
        four_point: bool = False,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["filename"] = filename
        self._params["four_point"] = four_point

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.get_data(
                self._params["filename"], self._params["four_point"]
            )
        )



from aamp_app.devices.device import Device
from .command import Command, CommandResult
from aamp_app.devices.linear_stage_150 import LinearStage150


class LinearStage150ParentCommand(Command):
    """Parent class for all LinearStage150 commands."""

    receiver_cls = LinearStage150

    def __init__(self, receiver: LinearStage150, **kwargs):
        super().__init__(receiver, **kwargs)


class LinearStage150Connect(LinearStage150ParentCommand):
    """Open a serial port for the linear stage."""

    def __init__(self, receiver: LinearStage150, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.start_serial())


class LinearStage150Initialize(LinearStage150ParentCommand):
    """Initialize the linear stage by homing it."""

    def __init__(self, receiver: LinearStage150, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class LinearStage150Deinitialize(LinearStage150ParentCommand):
    """Deinitialize the linear stage."""

    def __init__(self, receiver: LinearStage150, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.deinitialize())


class LinearStage150EnableMotor(LinearStage150ParentCommand):
    """Enable the linear stage motor."""

    def __init__(self, receiver: LinearStage150, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["state"] = True

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.set_enabled_state(self._params["state"])
        )


class LinearStage150DisableMotor(LinearStage150ParentCommand):
    """Disable the linear stage motor."""

    def __init__(self, receiver: LinearStage150, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["state"] = False

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.set_enabled_state(self._params["state"])
        )


class LinearStage150MoveAbsolute(LinearStage150ParentCommand):
    """Move the linear stage to an absolute position."""

    def __init__(self, receiver: LinearStage150, position: float, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["position"] = position

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.move_absolute(self._params["position"])
        )


class LinearStage150MoveRelative(LinearStage150ParentCommand):
    """Move the linear stage by a relative distance."""

    def __init__(self, receiver: LinearStage150, distance: float, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["distance"] = distance

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.move_relative(self._params["distance"])
        )



from .command import Command, CommandResult, CompositeCommand
from aamp_app.devices.newport_esp301 import NewportESP301
from typing import Optional


# Parent class, subclass from Command ABC
class NewportESP301ParentCommand(Command):
    """Parent class for all NewportESP301 commands."""

    receiver_cls = NewportESP301

    def __init__(self, receiver: NewportESP301, **kwargs):
        super().__init__(receiver, **kwargs)


# Recommended command classes
class NewportESP301Connect(NewportESP301ParentCommand):
    """Open the serial port to the ESP301 controller."""

    def __init__(self, receiver: NewportESP301, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.start_serial())


class NewportESP301Initialize(NewportESP301ParentCommand):
    """Initialize the axes by homing them."""

    def __init__(self, receiver: NewportESP301, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class NewportESP301Deinitialize(NewportESP301ParentCommand):
    """Deinitialize the axes by moving them to position zero."""

    def __init__(self, receiver: NewportESP301, reset_init_flag: bool = True, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["reset_init_flag"] = reset_init_flag

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.deinitialize(self._params["reset_init_flag"])
        )


# Device-related commands classes
class NewportESP301MoveSpeedAbsolute(NewportESP301ParentCommand):
    """Move axis to absolute position at specific speed (No speed uses default speed)."""

    def __init__(
        self,
        receiver: NewportESP301,
        axis_number: int = 1,
        position: Optional[float] = None,
        speed: Optional[float] = None,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["position"] = position
        self._params["speed"] = speed
        self._params["axis_number"] = axis_number

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.move_speed_absolute(
                self._params["axis_number"],
                self._params["position"],
                self._params["speed"],
            )
        )


class NewportESP301MoveSpeedRelative(NewportESP301ParentCommand):
    """Move axis by relative distance at specific speed (No speed uses default speed)."""

    def __init__(
        self,
        receiver: NewportESP301,
        axis_number: int = 1,
        distance: Optional[float] = None,
        speed: Optional[float] = None,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["distance"] = distance
        self._params["speed"] = speed
        self._params["axis_number"] = axis_number

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.move_speed_relative(
                self._params["axis_number"],
                self._params["distance"],
                self._params["speed"],
            )
        )


# Example of command with additional logic to determine the returned tuple of (success/fail: bool, success/fail message: str)
class NewportESP301SetDefaultSpeed(NewportESP301ParentCommand):
    """Set the default speed of the axes."""

    def __init__(self, receiver: NewportESP301, speed: float, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["speed"] = speed

    def execute(self) -> None:
        self._receiver.default_speed = self._params["speed"]
        # receiver's default_speed has a setter than checks the value is > 0 and < max_speed before setting
        # therefore, we can check if we actually changed the receiver's default speed, if not, it means the speed was out of range
        if self._receiver.default_speed == self._params["speed"]:
            self._result = CommandResult(
                True, "Default speed successfully set to " + str(self._params["speed"])
            )
        else:
            self._result = CommandResult(
                False,
                "Failed to set the default speed to "
                + str(self._params["speed"])
                + ". The default speed must be > 0 and < "
                + str(self._receiver._max_speed),
            )


# Derived commands
class NewportESP301HorzMoveSpeedAbsolute(NewportESP301MoveSpeedAbsolute):
    """desc"""

    # position no longer can be None here because we dont need axis num in sig
    def __init__(
        self,
        receiver: NewportESP301,
        position: float,
        speed: Optional[float] = None,
        **kwargs
    ):
        super().__init__(receiver, 1, position, speed, **kwargs)


# Just for testing composite commands
class NewportESP301Dance(CompositeCommand):
    def __init__(self, receiver, speed, distance, **kwargs):
        super().__init__(**kwargs)
        self.add_command(NewportESP301MoveSpeedRelative(receiver, -distance, speed))
        self.add_command(NewportESP301MoveSpeedRelative(receiver, distance, speed))



from typing import Tuple, Optional

from .command import Command, CommandResult
from aamp_app.devices.stellarnet_spectrometer import StellarNetSpectrometer


class SpectrometerParentCommand(Command):
    """Parent class for all StellarNet Spectrometer commands."""

    receiver_cls = StellarNetSpectrometer

    def __init__(self, receiver: StellarNetSpectrometer, **kwargs):
        super().__init__(receiver, **kwargs)


class SpectrometerInitialize(SpectrometerParentCommand):
    """Initialize spectrometer by verifying connection and getting each spectrometer object and wavelength array."""

    def __init__(self, receiver: StellarNetSpectrometer, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class SpectrometerDeinitialize(SpectrometerParentCommand):
    """Deinitialize the spectrometer, currently does nothing except optionally change init flag."""

    def __init__(
        self, receiver: StellarNetSpectrometer, reset_init_flag: bool = True, **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["reset_init_flag"] = reset_init_flag

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.deinitialize(self._params["reset_init_flag"])
        )


class SpectrometerUpdateDark(SpectrometerParentCommand):
    """Update the stored dark spectra for all spectrometers."""

    def __init__(
        self,
        receiver: StellarNetSpectrometer,
        integration_times: Tuple[int, ...] = (100, 100),
        scans_to_avg: Tuple[int, ...] = (3, 3),
        smoothings: Tuple[int, ...] = (0, 0),
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["integration_times"] = integration_times
        self._params["scans_to_avg"] = scans_to_avg
        self._params["smoothings"] = smoothings

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.update_all_dark_spectra(
                self._params["integration_times"],
                self._params["scans_to_avg"],
                self._params["smoothings"],
            )
        )


class SpectrometerUpdateBlank(SpectrometerParentCommand):
    """Update the stored blank spectra for all spectrometers."""

    def __init__(
        self,
        receiver: StellarNetSpectrometer,
        integration_times: Tuple[int, ...] = (100, 100),
        scans_to_avg: Tuple[int, ...] = (3, 3),
        smoothings: Tuple[int, ...] = (0, 0),
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["integration_times"] = integration_times
        self._params["scans_to_avg"] = scans_to_avg
        self._params["smoothings"] = smoothings

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.update_all_blank_spectra(
                self._params["integration_times"],
                self._params["scans_to_avg"],
                self._params["smoothings"],
            )
        )


class SpectrometerGetAbsorbance(SpectrometerParentCommand):
    """Calculate and update the stored absorbance spectra, merge spectra, and optionally save to file. No filename = timestamped filename."""

    def __init__(
        self,
        receiver: StellarNetSpectrometer,
        save_to_file: bool = True,
        filename: Optional[str] = None,
        integration_times: Tuple[int, ...] = (100, 100),
        scans_to_avg: Tuple[int, ...] = (3, 3),
        smoothings: Tuple[int, ...] = (0, 0),
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["save_to_file"] = save_to_file
        self._params["filename"] = filename
        self._params["integration_times"] = integration_times
        self._params["scans_to_avg"] = scans_to_avg
        self._params["smoothings"] = smoothings

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.get_all_absorbance(
                self._params["save_to_file"],
                self._params["filename"],
                self._params["integration_times"],
                self._params["scans_to_avg"],
                self._params["smoothings"],
            )
        )


class SpectrometerShutterIn(SpectrometerParentCommand):
    pass


class SpectrometerShutterOut(SpectrometerParentCommand):
    pass


class SpectrometerLampOn(SpectrometerParentCommand):
    pass


class SpectrometerLampOff(SpectrometerParentCommand):
    pass



from dash import Dash, html, dcc, dash_table, Input, Output, callback, State
import dash_bootstrap_components as dbc
import dash
import logging


dash.register_page(__name__, path="/execute-recipe", name="Execute Recipe", title="Execute Recipe")

layout = html.Div(
    [
        html.H1("Execute Recipe"),
        dbc.ButtonGroup(
            [
                dbc.Button("Execute", id="execute-button", n_clicks=0),
                dbc.Button("Clear Log", id="reset-button", n_clicks=0),
                dbc.Button("Emergency Stop", id="stop-button", n_clicks=0, color="danger"),
            ],
            className="mb-3",
        ),
        dbc.Row(
            [
                dbc.Col(
                    [
                        dbc.Input(
                            id="execute-recipe-upload-name",
                            placeholder="Name this execution",
                            type="text",
                        ),
                    ]
                )
            ],
            className="mb-3",
        ),
        # html.Div(
        #     [
        #         dbc.Switch(
        #             id='show-log-switch',
        #             label='Show log',
        #             value = False,
        #             style={'display': 'none'}
        #         )
        #     ],
        #     className="d-flex align-items-center mt-3",
        # ),
        html.Div(id="execute-recipe-output", className="mt-3", style={"display": "none"}),
        dcc.Interval(id="update-interval", interval=500, n_intervals=0),
        dcc.Interval(id="interval1", interval=50, n_intervals=0),
        html.Div(id="hidden-div", style={"display": "none"}),
        dbc.Row(
            [
                dbc.Col(
                    [
                        html.H4("Recipe Data"),
                        dbc.Textarea(
                            id="execute-recipe-upload-document",
                            className="log-container mb-3",
                            readOnly=True,
                            style={"height": "200px"},
                        ),
                    ]
                ),
            ]
        ),
        html.H4("Log"),
        html.Div(
            id="console-out2",
            className="log-container mb-3",
            style={
                "height": "500px",
                "overflow-y": "scroll",
                "padding": "10px",
                "border": "2px solid",
            },
        ),
        dbc.Row(
            [
                dbc.Col(
                    [
                        html.H4("Notes"),
                        dbc.Textarea(
                            id="execute-recipe-upload-notes",
                            className="log-container mb-3",
                            style={"height": "200px"},
                        ),
                    ]
                ),
                dbc.Col(
                    [
                        html.H4("Upload Files"),
                        dcc.Upload(
                            id="execute-recipe-upload-files",
                            children=html.Div(
                                [
                                    "Drag and Drop or ",
                                    html.A(
                                        "Select Files",
                                        style={
                                            "color": "blue",
                                            "textDecoration": "underline",
                                        },
                                    ),
                                    " or ",
                                    html.A(
                                        "Replace Selected",
                                        id="execute-recipe-clear-files",
                                        style={
                                            "color": "blue",
                                            "textDecoration": "underline",
                                        },
                                    ),
                                    html.Div(id="execute-recipe-upload-files-names"),
                                ]
                            ),
                            style={
                                "width": "100%",
                                "height": "200px",
                                "lineHeight": "60px",
                                "borderWidth": "1px",
                                "borderStyle": "dashed",
                                "borderRadius": "5px",
                                "textAlign": "center",
                            },
                            multiple=True,
                        ),
                    ]
                ),
            ]
        ),
        dbc.Button(
            "Save and Upload Data",
            id="execute-recipe-upload-data-button",
            className="mb-3",
            color="primary",
        ),
        html.Div(id="execute-recipe-upload-data-output"),
    ],
    className="container",
)


@callback(
    Output("execute-recipe-upload-files-names", "children"),
    Input("execute-recipe-upload-files", "filename"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def add_filenames_to_upload_box(filenames, url):
    if str(url) == "/execute-recipe":
        if filenames is None or filenames == []:
            return ""
        toRet = ""
        for i, filename in enumerate(filenames):
            if i < len(filenames) - 1:
                toRet += filename + ", "
            else:
                toRet += filename
        return toRet


@callback(
    Output("execute-recipe-upload-files", "filename"),
    Input("execute-recipe-clear-files", "n_clicks"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def clear_selected_files(n, url):
    if str(url) == "/execute-recipe":
        return None



from typing import Tuple, List
import asyncio
from aamp_app.devices.mfc import MassFlowController
from .command import Command, CommandResult


class MFCParentCommand(Command):
    """Parent class for MKS instruments mass flow controller"""

    receiver_cls = MassFlowController

    def __init__(self, receiver: MassFlowController, **kwargs):
        super().__init__(receiver, **kwargs)


class MFCInitialize(MFCParentCommand):
    """Initialize the mass flow controller"""

    def __init__(self, receiver: MassFlowController, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class MFCDeinitialize(MFCParentCommand):
    """Deinitialize the mass flow controller"""

    def __init__(self, receiver: MassFlowController, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(
            asyncio.run(self._receiver.deinitialize()), "Deinitialized the mfc"
        )
        # self._result = CommandResult(*self._receiver.deinitialize())


class MFCGetData(MFCParentCommand):
    """Return data on mfc operation"""

    def __init__(self, receiver: MassFlowController, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        # asyncio.run(self._receiver.get())
        self._result = CommandResult(asyncio.run(self._receiver.get()), "Received Data")
        # self._result = CommandResult(True, "got data")


class MFCSetGas(MFCParentCommand):
    """Set the type of gas for operation"""

    """Gas instance must first be created within web browser"""

    def __init__(self, receiver: MassFlowController, gas: str, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["gas"] = gas

    def execute(self) -> None:
        self._result = CommandResult(
            asyncio.run(self._receiver.set_gas(self._params["gas"])), "Set the gas"
        )
        # self._result = CommandResult(*self._receiver.set_gas(self._params['gas']))


class MFCSet(MFCParentCommand):
    """Set the desired flowrate in sccm"""

    def __init__(self, receiver: MassFlowController, setpoint: int, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["setpoint"] = setpoint

    def execute(self) -> None:
        self._result = CommandResult(
            asyncio.run(self._receiver.set(self._params["setpoint"])),
            "Set the flowrate",
        )
        # self._result = CommandResult(*self._receiver.set(self._params['setpoint']))


class MFCOpen(MFCParentCommand):
    """Open the mfc, set the flowrate to the maximum for the set gas"""

    def __init__(self, receiver: MassFlowController, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(
            asyncio.run(self._receiver.open()), "Opened the mfc, maximum flowrate"
        )
        # self._result = CommandResult(*self._receiver.open())



from dash import Dash, html, dcc, dash_table, Input, Output, callback
import dash_bootstrap_components as dbc
import dash


dash.register_page(__name__, path="/manual-control", title="Manual Control", name="Manual Control")

layout = html.Div(
    [
        html.H1("Manual Control", className="mb-3"),
        dcc.Interval(id="interval-manual-control", interval=500000, n_intervals=0),
        dbc.ButtonGroup(
            [
                dbc.Button(
                    "Open Execute Window",
                    id="manual-control-open-execute-modal-button",
                    n_clicks=0,
                    disabled=True,
                ),
                dbc.Button("Clear", id="manual-control-clear-button", n_clicks=0, disabled=True),
            ],
            className="mb-3",
        ),
        dbc.Modal(
            [
                dbc.ModalHeader(dbc.ModalTitle("Execute")),
                dbc.ModalBody(
                    [
                        dbc.Row(
                            [
                                dbc.Col(
                                    [
                                        dbc.Button(
                                            "Execute",
                                            id="manual-control-execute-button",
                                            n_clicks=0,
                                            style={"width": "100%"},
                                        )
                                    ],
                                    width=2,
                                ),
                                dbc.Col(
                                    [
                                        dbc.Alert(
                                            "Alert",
                                            id="manual-control-alert",
                                            is_open=False,
                                            duration=500,
                                        )
                                    ]
                                ),
                            ]
                        ),
                        html.Div(
                            id="manual-control-execute-modal-body-code",
                            className="log-container",
                            style={
                                "height": "160px",
                                "overflow-y": "scroll",
                                "padding": "10px",
                                "border": "2px solid",
                                "margin-bottom": "10px",
                            },
                        ),
                        html.Div(
                            id="manual-control-execute-modal-body",
                            className="log-container",
                            style={
                                "height": "300px",
                                "overflow-y": "scroll",
                                "padding": "10px",
                                "border": "2px solid",
                            },
                        ),
                    ]
                ),
            ],
            id="manual-control-execute-modal",
            size="xl",
        ),
        dbc.Modal(
            [
                dbc.ModalHeader(dbc.ModalTitle("COM Port Information")),
                dbc.ModalBody([html.Div(id="manual-control-serial-ports-info")]),
            ],
            id="manual-control-port-modal",
        ),
        dbc.Row(
            [
                dbc.Col(
                    [
                        dcc.Dropdown(
                            id="manual-control-device-dropdown",
                            options=[],
                            value=None,
                            className="mb-3",
                        ),
                        dbc.Col(
                            [
                                dbc.NavLink(
                                    id="manual-control-port-field",
                                    style={"display": "none"},
                                ),
                            ],
                            id="manual-control-device-form",
                        ),
                    ]
                ),
                dbc.Col(
                    [
                        dcc.Dropdown(
                            id="manual-control-command-dropdown",
                            options=[],
                            value=None,
                            disabled=True,
                            className="mb-3",
                        ),
                        dbc.Col([], id="manual-control-command-form"),
                    ]
                ),
            ],
        ),
    ],
    className="container",
)



from typing import Tuple, List
from aamp_app.devices.sht85_sensor import SHT85HumidityTempSensor
from .command import Command, CommandResult


class SHT85ParentCommand(Command):
    """Parent class for SHT85 humidity and temperature sensor"""

    receiver_cls = SHT85HumidityTempSensor

    def __init__(self, receiver: SHT85HumidityTempSensor, **kwargs):
        super().__init__(receiver, **kwargs)


class SHT85Connect(Command):
    """Open the serial port to the SHT85 sensor."""

    receiver_cls = SHT85HumidityTempSensor

    def __init__(self, receiver: SHT85HumidityTempSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.start_serial())


class SHT85Initialize(SHT85ParentCommand):
    """Initialize the humidity and temperature sensor"""

    def __init__(self, receiver: SHT85HumidityTempSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class SHT85Deinitialize(SHT85ParentCommand):
    """Deinitialize the humidity and temperature sensor"""

    def __init__(self, receiver: SHT85HumidityTempSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.deinitialize())


class SHT85GetHumidity(SHT85ParentCommand):
    """Read humidity data"""

    def __init__(self, receiver: SHT85HumidityTempSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.get_humidity())


class SHT85GetTemp(SHT85ParentCommand):
    """Read temperature data"""

    def __init__(self, receiver: SHT85HumidityTempSensor, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.get_temp())



from typing import Tuple
import time
from mfc import FlowController
import asyncio
from .device import Device, check_initialized


# Default ip address is 192.168.2.155
# Secondary mfc ip address should be set to 192.168.2.156 and so on
# Must pip install the correct edited version of mfc python library
# +24 VDC is pin 7, ground is pin 5


class MassFlowController(Device):
    def __init__(self, name: str, ip: str):
        super().__init__(name)
        self._ip = ip

    def initialize(self) -> Tuple[bool, str]:
        self._is_initialized = True
        return (True, "Initialized mass flow controller")

    async def deinitialize(self) -> Tuple[bool, str]:
        async with FlowController(self._ip) as fc:
            await fc.disconnect()
            self._is_initialized = False
        return (True, "Deinitialized mass flow controller")

    @check_initialized
    async def get(self) -> Tuple[bool, dict]:
        async with FlowController(self._ip) as fc:
            info = await fc.get()
            return (True, info)

    @check_initialized
    async def set_gas(self, gas: str) -> Tuple[bool, str]:
        """Gas instance must first be created within web browser interface"""
        async with FlowController(self._ip) as fc:
            await fc.set_gas(gas)
        return (True, f"Set MFC gas to {gas}")

    @check_initialized
    async def set(self, setpoint) -> Tuple[bool, str]:
        async with FlowController(self._ip) as fc:
            await fc.set(setpoint)
        return (True, f"Set MFC flowrate to {setpoint} sccm")

    async def open(self) -> Tuple[bool, str]:
        async with FlowController(self._ip) as fc:
            await fc.open()
        return (True, "Set the MFC flowrate to its maximum")



from .command import Command, CommandResult
from aamp_app.devices.psd6_syringe_pump import PSD6SyringePump
from typing import Optional


# Parent class, subclass from Command ABC
class PSD6SyringePumpParentCommand(Command):
    """Parent class for all PSD6SyringePump commands."""

    receiver_cls = PSD6SyringePump

    def __init__(self, receiver: PSD6SyringePump, **kwargs):
        super().__init__(receiver, **kwargs)


# Recommended command classes
class PSD6SyringePumpConnect(PSD6SyringePumpParentCommand):
    """Open the serial port to the PSD6 syringe pump."""

    def __init__(self, receiver: PSD6SyringePump, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.start_serial())


class PSD6SyringePumpInitialize(PSD6SyringePumpParentCommand):
    def __init__(self, receiver: PSD6SyringePump, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class PSD6SyringePumpMoveValve(PSD6SyringePumpParentCommand):
    def __init__(self, receiver: PSD6SyringePump, valve_num: int, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["valve_num"] = valve_num

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.move_valve_position(self._params["valve_num"])
        )


class PSD6SyringePumpMoveAbsolute(PSD6SyringePumpParentCommand):
    def __init__(
        self,
        receiver: PSD6SyringePump,
        volume: float,
        valve_num: Optional[int] = None,
        flowrate: Optional[float] = None,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["volume"] = volume
        self._params["valve_num"] = valve_num
        self._params["flowrate"] = flowrate

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.move_syringe_absolute_volume(
                self._params["volume"],
                self._params["valve_num"],
                self._params["flowrate"],
            )
        )


class PSD6SyringePumpInfuse(PSD6SyringePumpParentCommand):
    def __init__(
        self,
        receiver: PSD6SyringePump,
        volume: float,
        valve_num: Optional[int] = None,
        flowrate: Optional[float] = None,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["volume"] = volume
        self._params["valve_num"] = valve_num
        self._params["flowrate"] = flowrate

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.infuse_syringe_volume(
                self._params["volume"],
                self._params["valve_num"],
                self._params["flowrate"],
            )
        )


class PSD6SyringePumpWithdraw(PSD6SyringePumpParentCommand):
    def __init__(
        self,
        receiver: PSD6SyringePump,
        volume: float,
        valve_num: Optional[int] = None,
        flowrate: Optional[float] = None,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["volume"] = volume
        self._params["valve_num"] = valve_num
        self._params["flowrate"] = flowrate

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.withdraw_syringe_volume(
                self._params["volume"],
                self._params["valve_num"],
                self._params["flowrate"],
            )
        )



from dash import Dash, html, dcc, Input, Output, callback
import dash_bootstrap_components as dbc
import dash
from aamp_app.util import devices_ref_redundancy

dash.register_page(
    __name__,
    path="/real-time-telemetry",
    name="Real Time Telemetry",
    title="Real Time Telemetry",
)

layout = html.Div(
    [
        html.H1("Real Time Telemetry", className="mb-3"),
        dcc.Dropdown(
            options=list(devices_ref_redundancy.keys()),
            id="real-time-telemetry-device-dropdown",
        ),
        dcc.Interval(id="interval-real-time-telemetry", interval=500, n_intervals=0),
        html.Div(id="real-time-telemetry-div"),
    ],
    className="container",
)



from .command import Command, CommandResult
from aamp_app.devices.heating_stage import HeatingStage


# parent class for all HeatingStage commands
class HeatingStageParentCommand(Command):
    """Parent class for all HeatingStage commands."""

    receiver_cls = HeatingStage

    def __init__(self, receiver: HeatingStage, **kwargs):
        super().__init__(receiver, **kwargs)


class HeatingStageConnect(HeatingStageParentCommand):
    """Open serial port of heating stage's arduino controller."""

    def __init__(self, receiver: HeatingStage, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        # the serial port parameters should already be set in the HeatingStage instance
        self._result = CommandResult(*self._receiver.start_serial())


class HeatingStageInitialize(HeatingStageParentCommand):
    """Initialize heating stage by setting to room temperature and turning PID ON."""

    def __init__(self, receiver: HeatingStage, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class HeatingStageDeinitialize(HeatingStageParentCommand):
    """Deinitialize heating stage by setting to room temperature and turning PID OFF."""

    def __init__(self, receiver: HeatingStage, reset_init_flag: bool = True, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["reset_init_flag"] = reset_init_flag

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.deinitialize(self._params["reset_init_flag"])
        )


class HeatingStageSetTemp(HeatingStageParentCommand):
    """Set heating stage temperature. Execution completes when stabilized."""

    def __init__(self, receiver: HeatingStage, temperature: float, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["temperature"] = temperature

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.set_temp(self._params["temperature"])
        )


class HeatingStageSetSetPoint(HeatingStageParentCommand):
    """Set heating stage setpoint temperature. Execution completes immediately after setting setpoint, even if temperature is not stabilized."""

    def __init__(self, receiver: HeatingStage, temperature: float, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["temperature"] = temperature

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.set_settemp(self._params["temperature"])
        )



from .command import Command, CompositeCommand
from aamp_app.devices.dummy_heater import DummyHeater
from aamp_app.devices.dummy_motor import DummyMotor
from .dummy_heater_commands import *
from .dummy_motor_commands import *


# Example of a composite command that interacts with multiple devices (receivers)
class DummyPrinterHeatMove(CompositeCommand):
    """Sets DummyHeater to temperature, moves DummyMotor a relative distance and back, and sets DummyHeater to 25, reverts heat_rate and speed to original values"""

    def __init__(
        self,
        heater: DummyHeater,
        motor: DummyMotor,
        temp: float,
        heat_rate: float,
        distance: float,
        speed: float,
        **kwargs
    ):
        super().__init__(**kwargs)
        initial_heat_rate = heater.heat_rate
        initial_speed = motor.speed
        self.add_command(DummyHeaterSetHeatRate(heater, heat_rate))
        self.add_command(DummyHeaterSetTemp(heater, temp))
        self.add_command(DummyMotorSetSpeed(motor, speed))
        self.add_command(DummyMotorMoveRelative(motor, distance))
        self.add_command(DummyMotorMoveRelative(motor, -distance))
        self.add_command(DummyHeaterSetTemp(heater, 25))
        self.add_command(DummyHeaterSetHeatRate(heater, initial_heat_rate))
        self.add_command(DummyMotorSetSpeed(motor, initial_speed))



from .device import ArduinoSerialDevice, SerialDevice, check_initialized, check_serial
import serial
from time import sleep
from typing import Optional, Tuple


class SHT85HumidityTempSensor(ArduinoSerialDevice):
    """Humidity and Temperature Sensor"""

    def __init__(
        self, name: str, port: str, baudrate: int = 9600, timeout: Optional[float] = 1.0
    ):
        super().__init__(name, port, baudrate, timeout)
        # self.ser = serial.Serial()
        # self.ser.bytesize = serial.EIGHTBITS
        # self.ser.parity = serial.PARITY_NONE

    def initialize(self) -> Tuple[bool, str]:
        # self.ser.setDTR(False)
        # self.ser.flushInput()
        # self.ser.setDTR(True)
        # self.ser.open()
        self._is_initialized = True
        return (True, "Initialized humidity and temperature sensor")

    def deinitialize(self) -> Tuple[bool, str]:
        self.ser.close()
        self._is_initialized = False
        return (True, "Deinitialized humidity and temperature sensor")

    @check_serial
    @check_initialized
    def get_humidity(self) -> Tuple[bool, float, str]:
        self.ser.write(b"H")
        sleep(0.75)
        humidity = float(self.ser.readline().strip().decode())
        return (True, f"The relative humidity is {humidity}%")

    @check_serial
    @check_initialized
    def get_temp(self) -> Tuple[bool, float, str]:
        self.ser.write(b"T")
        sleep(0.75)
        temp = float(self.ser.readline().strip().decode())
        return (True, f"The temperature is {temp} degrees C")



from typing import Optional

from .command import Command, CommandResult
from aamp_app.devices.ximea_camera import XimeaCamera


class XimeaCameraParentCommand(Command):
    """Parent class for all XimeaCamera commands."""

    receiver_cls = XimeaCamera

    def __init__(self, receiver: XimeaCamera, **kwargs):
        super().__init__(receiver, **kwargs)


class XimeaCameraInitialize(XimeaCameraParentCommand):
    """Initialize camera by opening communication with camera and optionally set defaults."""

    def __init__(self, receiver: XimeaCamera, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.initialize())


class XimeaCameraDeinitialize(XimeaCameraParentCommand):
    """Deinitialize camera by stopping any acquisition and closing communication with camera."""

    def __init__(self, receiver: XimeaCamera, reset_init_flag: bool = True, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["reset_init_flag"] = reset_init_flag

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.deinitialize(self._params["reset_init_flag"])
        )


class XimeaCameraGetImage(XimeaCameraParentCommand):
    """Get image and save to file, display image, or both. No filename = timestamped filename. No exposure or gain = use defaults."""

    def __init__(
        self,
        receiver: XimeaCamera,
        save_to_file: bool = True,
        filename: str = None,
        exposure_time: Optional[int] = None,
        gain: Optional[float] = None,
        show_pop_up: bool = False,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["save_to_file"] = save_to_file
        self._params["filename"] = filename
        self._params["exposure_time"] = exposure_time
        self._params["gain"] = gain
        self._params["show_pop_up"] = show_pop_up

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.get_image(
                self._params["save_to_file"],
                self._params["filename"],
                self._params["exposure_time"],
                self._params["gain"],
                self._params["show_pop_up"],
            )
        )


class XimeaCameraSetDefaultExposure(XimeaCameraParentCommand):
    """Set the default exposure time to use if no exposure time is passed when getting image."""

    def __init__(self, receiver: XimeaCamera, exposure_time: int, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["exposure_time"] = exposure_time

    def execute(self) -> None:
        self._receiver.default_exposure_time = self._params["exposure_time"]
        if self._receiver.default_exposure_time == self._params["exposure_time"]:
            self._result = CommandResult(
                True,
                "Default exposure time successfully set to "
                + str(self._params["exposure_time"]),
            )
        else:
            self._result = CommandResult(
                False,
                "Failed to set the default exposure time to "
                + str(self._params["exposure_time"])
                + ". The default exposure time must be > 0.",
            )


class XimeaCameraSetDefaultGain(XimeaCameraParentCommand):
    """Set the default gain to use if no gain is passed when getting image."""

    def __init__(self, receiver: XimeaCamera, gain: float, **kwargs):
        super().__init__(receiver, **kwargs)
        self._params["gain"] = gain

    def execute(self) -> None:
        self._receiver.default_gain = self._params["gain"]
        if self._receiver.default_gain == self._params["gain"]:
            self._result = CommandResult(
                True, "Default gain successfully set to " + str(self._params["gain"])
            )
        else:
            # unsure what the upper limit is at the moment
            self._result = CommandResult(
                False,
                "Failed to set the default gain to "
                + str(self._params["gain"])
                + ". The default gain must be >= 0 and has an upper limit.",
            )


class XimeaCameraUpdateWhiteBal(XimeaCameraParentCommand):
    """Update the white balance coefficients using the current camera feed."""

    def __init__(
        self,
        receiver: XimeaCamera,
        exposure_time: int = None,
        gain: Optional[float] = None,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["exposure_time"] = exposure_time
        self._params["gain"] = gain

    def execute(self) -> None:
        self._result = CommandResult(
            *self._receiver.update_white_balance(
                self._params["exposure_time"], self._params["gain"]
            )
        )


class XimeaCameraSetManualWhiteBal(XimeaCameraParentCommand):
    """Set any of the red, green, and blue white balance coefficients manually."""

    def __init__(
        self,
        receiver: XimeaCamera,
        wb_kr: Optional[float] = None,
        wb_kg: Optional[float] = None,
        wb_kb: Optional[float] = None,
        **kwargs
    ):
        super().__init__(receiver, **kwargs)
        self._params["wb_kr"] = wb_kr
        self._params["wb_kg"] = wb_kg
        self._params["wb_kb"] = wb_kb

    def execute(self) -> None:
        self._result = CommandResult(
            self._receiver.set_white_balance_manually(
                self._params["wb_kr"], self._params["wb_kg"], self._params["wb_kb"]
            )
        )


class XimeaCameraResetWhiteBal(XimeaCameraParentCommand):
    """Reset the red, green, and blue white balance coefficients to defaults."""

    def __init__(self, receiver: XimeaCamera, **kwargs):
        super().__init__(receiver, **kwargs)

    def execute(self) -> None:
        self._result = CommandResult(*self._receiver.reset_white_balance_rgb_coeffs())
